//****************************************************************************//
//                                                                            //
//   recola.hpp                                                               //
//   is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2       //
//                                                                            //
//   Copyright (C) 2015-2017 Ansgar Denner, Jean-Nicolas Lang and             //
//                           Sandro Uccirati                                  //
//                                                                            //
//   C++ interface developed by Benedikt Biedermann and Mathieu Pellen        //
//                                                                            //
//   RECOLA2 is licenced under the GNU GPL version 3,                         //
//   see COPYING for details.                                                 //
//                                                                            //
//****************************************************************************//

#ifndef RECOLA_HPP_
#define RECOLA_HPP_

#include <complex>
struct dcomplex{double dr,di;};

#define ivoid inline void

#ifdef __cplusplus
extern "C" {
#endif
  // module input_rcl
  void __input_rcl_MOD_set_qcd_rescaling_rcl
       (const int*);
  void __input_rcl_MOD_set_fermionloop_optimization_rcl
       (const int*);
  void __input_rcl_MOD_set_helicity_optimization_rcl
       (const int*);
  void __input_rcl_MOD_set_colour_optimization_rcl
       (const int*);
  void __input_rcl_MOD_set_compute_A12_rcl
       (const int*);
  void __input_rcl_MOD_set_masscut_rcl
       (const double*);
  void __input_rcl_MOD_set_light_particle_rcl
       (const char*,long int);
  void __input_rcl_MOD_unset_light_particle_rcl
       (const char*,long int);
  void __input_rcl_MOD_use_dim_reg_soft_rcl
       ();
  void __input_rcl_MOD_use_mass_reg_soft_rcl
       (const double*);
  void __input_rcl_MOD_use_recola_base_rcl
       (const int*);
  void __input_rcl_MOD_set_mass_reg_soft_rcl
       (const double*);
  void __input_rcl_MOD_set_delta_uv_rcl
       (const double*);
  void __input_rcl_MOD_get_delta_uv_rcl
       (const double*);
  void __input_rcl_MOD_set_delta_ir_rcl
       (const double*, const double*);
  void __input_rcl_MOD_get_delta_ir_rcl
       (const double*, const double*);
  void __input_rcl_MOD_set_mu_uv_rcl
       (const double*);
  void __input_rcl_MOD_get_mu_uv_rcl
       (const double*);
  void __input_rcl_MOD_set_mu_ir_rcl
       (const double*);
  void __input_rcl_MOD_get_mu_ir_rcl
       (const double*);
  void __input_rcl_MOD_set_mu_ms_rcl
       (const double*);
  void __input_rcl_MOD_get_mu_ms_rcl
       (const double*);
  void __input_rcl_MOD_get_renormalization_scale_rcl
       (double*);
  void __input_rcl_MOD_get_flavour_scheme_rcl
       (int*);
  void __input_rcl_MOD_set_complex_mass_scheme_rcl
       ();
  void __input_rcl_MOD_set_on_shell_scheme_rcl
       ();
  void __input_rcl_MOD_set_momenta_correction_rcl
       (const int*);
  void __input_rcl_MOD_set_print_level_amplitude_rcl
       (const int*);
  void __input_rcl_MOD_set_print_level_squared_amplitude_rcl
       (const int*);
  void __input_rcl_MOD_set_print_level_correlations_rcl
       (const int*);
  void __input_rcl_MOD_set_print_level_parameters_rcl
       (const int*);
  void __input_rcl_MOD_set_draw_level_branches_rcl
       (const int*);
  void __input_rcl_MOD_set_print_level_ram_rcl
       (const int*);
  void __input_rcl_MOD_scale_coupling3_rcl
       (dcomplex*,const char*,const char*, const char*,
        long int,long int,long int);
  void __input_rcl_MOD_scale_coupling4_rcl
       (dcomplex*, const char*, const char*, const char*,const char*,
        long int,long int,long int,long int);
  void __input_rcl_MOD_switchoff_coupling3_rcl
       (const char*,const char*,const char*,long int,long int,long int);
  void __input_rcl_MOD_switchoff_coupling4_rcl
       (const char*,const char*,const char*,const char*,
        long int,long int,long int,long int);
  void __input_rcl_MOD_switchon_resonant_selfenergies_rcl
       ();
  void __input_rcl_MOD_switchoff_resonant_selfenergies_rcl
       ();
  void __input_rcl_MOD_set_dynamic_settings_rcl
       (const int*);
  void __input_rcl_MOD_set_ifail_rcl
       (const int*);
  void __input_rcl_MOD_get_ifail_rcl
       (int*);
  void __input_rcl_MOD_set_iwarn_rcl
       (const int*);
  void __input_rcl_MOD_get_iwarn_rcl
       (int*);
  void __input_rcl_MOD_set_collier_output_dir_rcl
       (const char*,long int);
  void __input_rcl_MOD_set_output_file_rcl
       (const char*,long int);
  void __input_rcl_MOD_set_log_mem_rcl
       (const int*);
  void __input_rcl_MOD_set_init_collier_rcl
       (const int*);
  void __input_rcl_MOD_set_parameter_rcl
       (const char*, dcomplex*, long int);
  void __input_rcl_MOD_get_parameter_rcl
       (const char*, dcomplex*, long int);
  void __input_rcl_MOD_set_renoscheme_rcl
       (const char*, const char*, long int, long int);
  // get_renoscheme_rcl -> wrapper
  void __input_rcl_MOD_set_resonant_particle_rcl
       (const char*,long int);
  void __input_rcl_MOD_reset_vertices_rcl
       ();
  void __input_rcl_MOD_reset_couplings_rcl
       ();
  void __input_rcl_MOD_reset_ctcouplings_rcl
       ();
  void __input_rcl_MOD_print_collier_statistics_rcl
       ();
  
  // module recola1_interface_rcl
  void __recola1_interface_rcl_MOD_set_pole_mass_w_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_onshell_mass_w_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_z_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_onshell_mass_z_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_h_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_top_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_bottom_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_charm_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_strange_rcl
       (const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_up_rcl
       (const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_down_rcl
       (const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_tau_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_muon_rcl
       (const double*,const double*);
  void __recola1_interface_rcl_MOD_set_pole_mass_electron_rcl
       (const double*);
  void __recola1_interface_rcl_MOD_set_light_top_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_top_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_bottom_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_bottom_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_charm_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_charm_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_strange_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_strange_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_up_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_up_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_down_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_down_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_tau_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_tau_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_muon_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_muon_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_electron_rcl
       ();
  void __recola1_interface_rcl_MOD_unset_light_electron_rcl
       ();
  void __recola1_interface_rcl_MOD_set_light_fermions_rcl
       (const double*);
  // use_gfermi_scheme_rcl -> wrapper
  void __recola1_interface_rcl_MOD_use_alpha0_scheme_rcl
       (const double*);
  void __recola1_interface_rcl_MOD_use_alphaz_scheme_rcl
       (const double*);
  void __recola1_interface_rcl_MOD_get_alpha_rcl
       (double*);
  void __recola1_interface_rcl_MOD_set_alphas_rcl
       (const double*,const double*,const int*);
  void __recola1_interface_rcl_MOD_get_alphas_rcl
       (double*);
  void __recola1_interface_rcl_MOD_set_alphas_masses_rcl
       (const double*,const double*,const double*,
        const double*,const double*,const double*);
  void __recola1_interface_rcl_MOD_compute_running_alphas_rcl
       (const double*, const int*, const int*);
  void __recola1_interface_rcl_MOD_select_all_gs_powers_bornampl_rcl
       (const int*);
  void __recola1_interface_rcl_MOD_unselect_all_gs_powers_bornampl_rcl
       (const int*);
  void __recola1_interface_rcl_MOD_select_gs_power_bornampl_rcl
       (const int*, const int*);
  void __recola1_interface_rcl_MOD_unselect_gs_power_bornampl_rcl
       (const int*, const int*);
  void __recola1_interface_rcl_MOD_select_all_gs_powers_loopampl_rcl
       (const int*);
  void __recola1_interface_rcl_MOD_unselect_all_gs_powers_loopampl_rcl
       (const int*);
  void __recola1_interface_rcl_MOD_select_gs_power_loopampl_rcl
       (const int*, const int*);
  void __recola1_interface_rcl_MOD_unselect_gs_power_loopampl_rcl
       (const int*, const int*);
  void __recola1_interface_rcl_MOD_get_colour_correlation_r1_rcl
       (const int*, const int*, const int*, const int*, double*);
  void __recola1_interface_rcl_MOD_get_spin_colour_correlation_r1_rcl
       (const int*, const int*, const int*, const int*, double*);
  void __recola1_interface_rcl_MOD_get_spin_correlation_r1_rcl
       (const int*, const int*, double*);
  // set_gs_power_rcl -> wrapper
  // get_amplitude_r1_rcl -> wrapper
  /* get_*_amplitude_r1_rcl ?:  <24-07-17, Jean-Nicolas Lang> */
  /* get_*_correlation_r1_rcl ?:  <24-07-17, Jean-Nicolas Lang> */


  // module process_definition_rcl
  void __process_definition_rcl_MOD_define_process_rcl
       (const int*,const char*,const char*,long int,long int);
  void __process_definition_rcl_MOD_select_power_bornampl_rcl
       (const int*,const char*, const int*,long int);
  void __process_definition_rcl_MOD_unselect_power_bornampl_rcl
       (const int*,const char*,const int*,long int);
  void __process_definition_rcl_MOD_select_all_powers_bornampl_rcl
       (const int*);
  void __process_definition_rcl_MOD_unselect_all_powers_bornampl_rcl
       (const int*);
  void __process_definition_rcl_MOD_select_power_loopampl_rcl
       (const int*,const char*,const int*,long int);
  void __process_definition_rcl_MOD_unselect_power_loopampl_rcl
       (const int*,const char*,const int*,long int);
  void __process_definition_rcl_MOD_select_all_powers_loopampl_rcl
       (const int*);
  void __process_definition_rcl_MOD_unselect_all_powers_loopampl_rcl
       (const int*);
  void __process_definition_rcl_MOD_split_collier_cache_rcl
       (const int*,const int*);

  // module process_generation_rcl
  void __process_generation_rcl_MOD_generate_processes_rcl
       ();

  // module process_computation_rcl
  void __process_computation_rcl_MOD_set_resonant_squared_momentum_rcl
       (const int*, const int*, const double*);
  void __process_computation_rcl_MOD_rescale_process_rcl
       (const int*,const char*,double*,long int);
  void __process_computation_rcl_MOD_rescale_colour_correlation_rcl
       (const int*,const int*,const int*,double*);
  void __process_computation_rcl_MOD_rescale_all_colour_correlations_rcl
       (const int*);
  void __process_computation_rcl_MOD_rescale_spin_colour_correlation_rcl
    (const int*, const int*, const int*,const dcomplex[4], double*);
  void __process_computation_rcl_MOD_rescale_spin_correlation_rcl
    (const int*, const int*, const dcomplex[4], double*);
  void __process_computation_rcl_MOD_set_tis_required_accuracy_rcl
       (const double*);
  void __process_computation_rcl_MOD_get_tis_required_accuracy_rcl
       (double*);
  void __process_computation_rcl_MOD_set_tis_critical_accuracy_rcl
       (const double*);
  void __process_computation_rcl_MOD_get_tis_critical_accuracy_rcl
       (double*);
  void __process_computation_rcl_MOD_get_tis_accuracy_flag_rcl
       (int*);

  /* get_colour/helicity_configs:  <24-07-17, Jean-Nicolas Lang> */

  // module wrapper_rcl
  void __wrapper_rcl_MOD_set_alphas_masses_nowidtharg_rcl
       (const double*,const double*,const double*);
  void __wrapper_rcl_MOD_use_gfermi_scheme_noarg_rcl
       ();
  void __wrapper_rcl_MOD_use_gfermi_scheme_and_set_alpha_rcl
       (const double*);
  void __wrapper_rcl_MOD_use_gfermi_scheme_and_set_gfermi_rcl
       (const double*);
  void __wrapper_rcl_MOD_use_alpha0_scheme_noarg_rcl
       ();
  void __wrapper_rcl_MOD_use_alphaz_scheme_noarg_rcl
       ();
  void __wrapper_rcl_MOD_wrapper_set_gs_power_rcl
       (const int*,const int[][2],const int*);
  void __wrapper_rcl_MOD_wrapper_set_outgoing_momenta_rcl
       (const int*, double[][4], double[][4], const int*);
  void __wrapper_rcl_MOD_wrapper_compute_process_rcl
       (const int*,const double[][4],const int*, 
        const char*,double*,int*,long int);
  void __wrapper_rcl_MOD_wrapper_get_amplitude_r1_rcl
       (const int*, const int*, const char*, const int[], const int[], 
        const int*, dcomplex*, long int);
  void __wrapper_rcl_MOD_wrapper_get_amplitude_general_rcl
       (const int*, const int[], const int*, const char*, const int[],
        const int[], const int*, dcomplex*, long int);
  void __wrapper_rcl_MOD_wrapper_get_squared_amplitude_r1_rcl
       (const int *, const int*, 
        const char*, double*, long int); 
  void __wrapper_rcl_MOD_wrapper_get_squared_amplitude_general_rcl
       (const int *, const int[], const int*,
        const char*, double*, long int); 
  void __wrapper_rcl_MOD_wrapper_get_polarized_squared_amplitude_r1_rcl
       (const int*, const int*, const char*, const int[], 
        const int*, double*, long int);
  void __wrapper_rcl_MOD_wrapper_get_polarized_squared_amplitude_general_rcl
       (const int*, const int[], const int*, const char*, const int[], 
        const int*, double*, long int);
  void __wrapper_rcl_MOD_wrapper_compute_colour_correlation_rcl
       (const int*, const double[][4], const int*, const int*, const int*,
        double*, int*);
  void __wrapper_rcl_MOD_wrapper_compute_all_colour_correlations_rcl
       (const int*, const double[][4], const int*, int*);
  void __wrapper_rcl_MOD_wrapper_get_colour_correlation_general_rcl
       (const int*, const int[], const int*, const int*,const int*, double*);
  void __wrapper_rcl_MOD_wrapper_compute_spin_colour_correlation_rcl
       (const int*, const double [][4], const int*, const int*, const int*,
        const dcomplex[4], double*, int*);
  void __wrapper_rcl_MOD_wrapper_get_spin_colour_correlation_general_rcl
       (const int*, const int[], const int*, const int*,const int*, double*);
  void __wrapper_rcl_MOD_wrapper_compute_spin_correlation_rcl
       (const int*, const double[][4], const int*, const int*, 
        const dcomplex[4], double*, int*);
  void __wrapper_rcl_MOD_wrapper_get_spin_correlation_general_rcl
       (const int*, const int[], const int*, double*);
  void __wrapper_rcl_MOD_get_legs_rcl
       (const int*, int*);
  void __wrapper_rcl_MOD_get_n_orders_rcl
       (int*);
  void __wrapper_rcl_MOD_wrapper_get_momenta_rcl
       (const int*, double [][4], const int*);
  void __wrapper_rcl_MOD_wrapper_get_recola_version_rcl
       (char*,int*,long int);
  void __wrapper_rcl_MOD_wrapper_get_modelname_rcl
       (char*,int*,long int);
  void __wrapper_rcl_MOD_wrapper_get_modelgauge_rcl
       (char*,int*,long int);
  void __wrapper_rcl_MOD_wrapper_get_driver_timestamp_rcl
       (char*,int*,long int);
  void __wrapper_rcl_MOD_wrapper_get_renoscheme_rcl
       (const char[],char[],int*,long int);

    // module extended_higgs_interface_rcl
  void __extended_higgs_interface_rcl_MOD_set_z2_thdm_yukawa_type_rcl
       (const int*);
  void __extended_higgs_interface_rcl_MOD_set_pole_mass_hl_rcl
       (const double*,const double*);
  void __extended_higgs_interface_rcl_MOD_set_pole_mass_hh_rcl
       (const double*,const double*);
  void __extended_higgs_interface_rcl_MOD_set_pole_mass_hl_hh_rcl
       (const double*,const double*,const double*,const double*);
  void __extended_higgs_interface_rcl_MOD_set_pole_mass_ha_rcl
       (const double*,const double*);
  void __extended_higgs_interface_rcl_MOD_set_pole_mass_hc_rcl
       (const double*,const double*);
  void __extended_higgs_interface_rcl_MOD_set_tb_cab_rcl
       (const double*,const double*);
  void __extended_higgs_interface_rcl_MOD_use_mixing_alpha_msbar_scheme_rcl
       (const char*,long int);
  void __extended_higgs_interface_rcl_MOD_use_mixing_alpha_onshell_scheme_rcl
       (const char*,long int);
  void __extended_higgs_interface_rcl_MOD_use_mixing_beta_onshell_scheme_rcl
       (const char*,long int);
  void __extended_higgs_interface_rcl_MOD_use_mixing_beta_msbar_scheme_rcl
       (const char*,long int);
  void __extended_higgs_interface_rcl_MOD_set_msb_rcl
       (const double*);
  void __extended_higgs_interface_rcl_MOD_use_msb_msbar_scheme_rcl
       (const char*,long int);
  void __extended_higgs_interface_rcl_MOD_set_sa_rcl
       (const double*);
  void __extended_higgs_interface_rcl_MOD_set_tb_rcl
       (const double*);
  void __extended_higgs_interface_rcl_MOD_use_tb_msbar_scheme_rcl
       (const char*,long int);

  // module statistics_rcl
  void __statistics_rcl_MOD_print_generation_statistics_rcl
       ();
  void __statistics_rcl_MOD_print_ti_statistics_rcl
       (const int*,const int*);
  void __statistics_rcl_MOD_print_tc_statistics_rcl
       (const int*,const int*);

  // module reset_rcl
  void __reset_rcl_MOD_reset_recola_rcl
       ();

#ifdef __cplusplus
}
#endif

namespace Recola{

/**********************
*  module input_rcl  *
**********************/

ivoid set_qcd_rescaling_rcl
      (const bool m)
{
  int boolint;
  if (m) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_qcd_rescaling_rcl(&boolint);
}
ivoid set_fermionloop_optimization_rcl(const bool m)
{
  int boolint;
  if (m) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_fermionloop_optimization_rcl(&boolint);
}
ivoid set_helicity_optimization_rcl(const bool m)
{
  int boolint;
  if (m) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_helicity_optimization_rcl(&boolint);
}
ivoid set_colour_optimization_rcl(const int m)
{
  __input_rcl_MOD_set_colour_optimization_rcl(&m);
}
ivoid set_compute_A12_rcl(const bool m)
{
  int boolint;
  if (m) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_compute_A12_rcl(&boolint);
}
ivoid set_masscut_rcl(const double mc)
{
  __input_rcl_MOD_set_masscut_rcl(&mc);
}
ivoid set_light_particle_rcl
      (const std::string pa)
{
  __input_rcl_MOD_set_light_particle_rcl(pa.c_str(),pa.length());
}
ivoid unset_light_particle_rcl
      (const std::string pa)
{
  __input_rcl_MOD_unset_light_particle_rcl(pa.c_str(),pa.length());
}
ivoid use_dim_reg_soft_rcl()
{
  __input_rcl_MOD_use_dim_reg_soft_rcl();
}
ivoid use_mass_reg_soft_rcl
      (const double m)
{
  __input_rcl_MOD_use_mass_reg_soft_rcl(&m);
}
ivoid use_recola_base_rcl
      (const int rb)
{
  __input_rcl_MOD_use_recola_base_rcl(&rb);
}
ivoid set_mass_reg_soft_rcl
      (const double m)
{
  __input_rcl_MOD_set_mass_reg_soft_rcl(&m);
}
ivoid set_delta_uv_rcl
      (const double d)
{
  __input_rcl_MOD_set_delta_uv_rcl(&d);
}
ivoid get_delta_uv_rcl
      (double &d)
{
  __input_rcl_MOD_get_delta_uv_rcl(&d);
}
ivoid set_delta_ir_rcl
      (const double d1, const double d2)
{
  __input_rcl_MOD_set_delta_ir_rcl(&d1,&d2);
}
ivoid get_delta_ir_rcl
      (double &d1, double &d2)
{
  __input_rcl_MOD_get_delta_ir_rcl(&d1,&d2);
}
ivoid set_mu_uv_rcl
      (const double m)
{
  __input_rcl_MOD_set_mu_uv_rcl(&m);
}
ivoid get_mu_uv_rcl
      (double &m)
{
  __input_rcl_MOD_get_mu_uv_rcl(&m);
}
ivoid set_mu_ir_rcl
      (const double m)
{
  __input_rcl_MOD_set_mu_ir_rcl(&m);
}
ivoid get_mu_ir_rcl
      (double &m)
{
  __input_rcl_MOD_get_mu_ir_rcl(&m);
}
ivoid set_mu_ms_rcl
      (const double m)
{
  __input_rcl_MOD_set_mu_ms_rcl(&m);
}
ivoid get_mu_ms_rcl
      (double &m)
{
  __input_rcl_MOD_get_mu_ms_rcl(&m);
}
ivoid get_renormalization_scale_rcl
      (double &mu)
{
  __input_rcl_MOD_get_renormalization_scale_rcl(&mu);
}
ivoid get_flavour_scheme_rcl
      (int &nf)
{
  __input_rcl_MOD_get_flavour_scheme_rcl(&nf);
}
ivoid set_complex_mass_scheme_rcl()
{
  __input_rcl_MOD_set_complex_mass_scheme_rcl();
}
ivoid set_on_shell_scheme_rcl()
{
  __input_rcl_MOD_set_on_shell_scheme_rcl();
}
ivoid set_momenta_correction_rcl
      (const bool mc)
{
  int boolint;
  if (mc) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_momenta_correction_rcl(&boolint);
}
ivoid set_print_level_amplitude_rcl
      (const int n)
{
  __input_rcl_MOD_set_print_level_amplitude_rcl(&n);
}
ivoid set_print_level_squared_amplitude_rcl
      (const int n)
{
  __input_rcl_MOD_set_print_level_squared_amplitude_rcl(&n);
}
ivoid set_print_level_correlations_rcl
      (const int n)
{
  __input_rcl_MOD_set_print_level_correlations_rcl(&n);
}
ivoid set_print_level_parameters_rcl
      (const int n)
{
  __input_rcl_MOD_set_print_level_parameters_rcl(&n);
}
ivoid set_draw_level_branches_rcl
      (const int n)
{
  __input_rcl_MOD_set_draw_level_branches_rcl(&n);
}
ivoid set_print_level_RAM_rcl
      (const int n)
{
  __input_rcl_MOD_set_print_level_ram_rcl(&n);
}
ivoid scale_coupling3_rcl
      (const std::complex<double> fac, 
       const std::string pa1,
       const std::string pa2, 
       const std::string pa3)
{
   dcomplex dfac;
   dfac.dr = fac.real();
   dfac.di = fac.imag();
   __input_rcl_MOD_scale_coupling3_rcl(&dfac,
                                       pa1.c_str(),
                                       pa2.c_str(),
                                       pa3.c_str(),
                                       pa1.length(),
                                       pa2.length(),
                                       pa3.length());
}
ivoid scale_coupling4_rcl
      (const std::complex<double> fac, 
       const std::string pa1,
       const std::string pa2, 
       const std::string pa3,
       const std::string pa4)
{
   dcomplex dfac;
   dfac.dr = fac.real();
   dfac.di = fac.imag();
   __input_rcl_MOD_scale_coupling4_rcl(&dfac,
                                       pa1.c_str(),
                                       pa2.c_str(),
                                       pa3.c_str(),
                                       pa4.c_str(),
                                       pa1.length(),
                                       pa2.length(),
                                       pa3.length(),
                                       pa4.length());
}
ivoid switchoff_coupling3_rcl
      (const std::string pa1,
       const std::string pa2,
       const std::string pa3)
{
  __input_rcl_MOD_switchoff_coupling3_rcl(pa1.c_str(),
                                          pa2.c_str(),
                                          pa3.c_str(),
                                          pa1.length(),
                                          pa2.length(),
                                          pa3.length());
}
ivoid switchoff_coupling4_rcl
      (const std::string pa1,
       const std::string pa2,
       const std::string pa3,
       const std::string pa4)
{
  __input_rcl_MOD_switchoff_coupling4_rcl(pa1.c_str(),
                                          pa2.c_str(),
                                          pa3.c_str(),
                                          pa4.c_str(),
                                          pa1.length(),
                                          pa2.length(),
                                          pa3.length(),
                                          pa4.length());
}
ivoid switchon_resonant_selfenergies_rcl()
{
  __input_rcl_MOD_switchon_resonant_selfenergies_rcl();
}
ivoid switchoff_resonant_selfenergies_rcl()
{
  __input_rcl_MOD_switchoff_resonant_selfenergies_rcl();
}
ivoid set_dynamic_settings_rcl
      (const int n)
{
  __input_rcl_MOD_set_dynamic_settings_rcl(&n);
}
ivoid set_ifail_rcl
      (const int i)
{
   __input_rcl_MOD_set_ifail_rcl(&i);
}
ivoid get_ifail_rcl
      (int &i)
{
   __input_rcl_MOD_get_ifail_rcl(&i);
}
ivoid set_iwarn_rcl
      (const int i)
{
   __input_rcl_MOD_set_iwarn_rcl(&i);
}
ivoid get_iwarn_rcl
      (int &i)
{
   __input_rcl_MOD_get_iwarn_rcl(&i);
}
ivoid set_collier_output_dir_rcl
      (const std::string x)
{
  __input_rcl_MOD_set_collier_output_dir_rcl(x.c_str(),x.length());
}
ivoid set_output_file_rcl
      (const std::string x)
{
  __input_rcl_MOD_set_output_file_rcl(x.c_str(),x.length());
}
ivoid set_log_mem_rcl
      (const bool lm)
{
  int boolint;
  if (lm) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_log_mem_rcl(&boolint);
}
ivoid set_init_collier_rcl
      (const bool ic)
{
  int boolint;
  if (ic) {
    boolint = 1;
  } else {
    boolint = 0;
  }
  __input_rcl_MOD_set_init_collier_rcl(&boolint);
}
ivoid set_parameter_rcl
      (const std::string pname, 
       std::complex<double> pvalue)
{
  dcomplex pvaluec;
  pvaluec.di = pvalue.imag();
  pvaluec.dr = pvalue.real();
  __input_rcl_MOD_set_parameter_rcl(pname.c_str(), 
                                    &pvaluec,
                                    pname.length());
}
ivoid get_parameter_rcl
      (const std::string pname, 
       std::complex<double>& pvalue)
{
  dcomplex pvaluec;
  __input_rcl_MOD_get_parameter_rcl(pname.c_str(), 
                                    &pvaluec,
                                    pname.length());
  pvalue = std::complex<double> (pvaluec.dr, pvaluec.di);
}
ivoid set_renoscheme_rcl
      (const std::string renoparam, 
       const std::string renoscheme)
{
  __input_rcl_MOD_set_renoscheme_rcl(renoparam.c_str(),
                                     renoscheme.c_str(),
                                     renoparam.length(),
                                     renoscheme.length());
}
ivoid set_resonant_particle_rcl
      (const std::string pa)
{
  __input_rcl_MOD_set_resonant_particle_rcl(pa.c_str(),pa.length());
}
ivoid reset_vertices_rcl
      ()
{
  __input_rcl_MOD_reset_vertices_rcl();
}
ivoid reset_couplings_rcl
      ()
{
  __input_rcl_MOD_reset_couplings_rcl();
}
ivoid reset_ctcouplings_rcl
      ()
{
  __input_rcl_MOD_reset_ctcouplings_rcl();
}
ivoid print_collier_statistics_rcl
      ()
{
  __input_rcl_MOD_print_collier_statistics_rcl();
}


/**********************************
*  module recola1_interface_rcl  *
**********************************/
ivoid set_pole_mass_w_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_w_rcl(&m,&g);
}
ivoid set_onshell_mass_w_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_onshell_mass_w_rcl(&m,&g);
}
ivoid set_pole_mass_z_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_z_rcl(&m,&g);
}
ivoid set_onshell_mass_z_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_onshell_mass_z_rcl(&m,&g);
}
ivoid set_pole_mass_h_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_h_rcl(&m,&g);
}
ivoid set_pole_mass_top_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_top_rcl(&m,&g);
}
ivoid set_pole_mass_bottom_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_bottom_rcl(&m,&g);
}
ivoid set_pole_mass_charm_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_charm_rcl(&m,&g);
}
ivoid set_pole_mass_strange_rcl
      (const double m)
{
  __recola1_interface_rcl_MOD_set_pole_mass_strange_rcl(&m);
}
ivoid set_pole_mass_up_rcl
      (const double m)
{
  __recola1_interface_rcl_MOD_set_pole_mass_up_rcl(&m);
}
ivoid set_pole_mass_down_rcl
      (const double m)
{
  __recola1_interface_rcl_MOD_set_pole_mass_down_rcl(&m);
}
ivoid set_pole_mass_tau_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_tau_rcl(&m,&g);
}
ivoid set_pole_mass_muon_rcl
      (const double m, const double g)
{
  __recola1_interface_rcl_MOD_set_pole_mass_muon_rcl(&m,&g);
}
ivoid set_pole_mass_electron_rcl
      (const double m)
{
  __recola1_interface_rcl_MOD_set_pole_mass_electron_rcl(&m);
}
ivoid set_light_top_rcl()
{
  __recola1_interface_rcl_MOD_set_light_top_rcl();
}
ivoid unset_light_top_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_top_rcl();
}
ivoid set_light_bottom_rcl()
{
  __recola1_interface_rcl_MOD_set_light_bottom_rcl();
}
ivoid unset_light_bottom_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_bottom_rcl();
}
ivoid set_light_charm_rcl()
{
  __recola1_interface_rcl_MOD_set_light_charm_rcl();
}
ivoid unset_light_charm_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_charm_rcl();
}
ivoid set_light_strange_rcl()
{
  __recola1_interface_rcl_MOD_set_light_strange_rcl();
}
ivoid unset_light_strange_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_strange_rcl();
}
ivoid set_light_up_rcl()
{
  __recola1_interface_rcl_MOD_set_light_up_rcl();
}
ivoid unset_light_up_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_up_rcl();
}
ivoid set_light_down_rcl()
{
  __recola1_interface_rcl_MOD_set_light_down_rcl();
}
ivoid unset_light_down_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_down_rcl();
}
ivoid set_light_tau_rcl()
{
  __recola1_interface_rcl_MOD_set_light_tau_rcl();
}
ivoid unset_light_tau_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_tau_rcl();
}
ivoid set_light_muon_rcl()
{
  __recola1_interface_rcl_MOD_set_light_muon_rcl();
}
ivoid unset_light_muon_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_muon_rcl();
}
ivoid set_light_electron_rcl()
{
  __recola1_interface_rcl_MOD_set_light_electron_rcl();
}
ivoid unset_light_electron_rcl()
{
  __recola1_interface_rcl_MOD_unset_light_electron_rcl();
}
ivoid set_light_fermions_rcl
      (const double m)
{
  __recola1_interface_rcl_MOD_set_light_fermions_rcl(&m);
}
ivoid use_alpha0_scheme_rcl
      (const double a)
{
  __recola1_interface_rcl_MOD_use_alpha0_scheme_rcl(&a);
}
ivoid use_alphaz_scheme_rcl
      (const double a)
{
  __recola1_interface_rcl_MOD_use_alphaz_scheme_rcl(&a);
}
ivoid get_alpha_rcl
      (double &a)
{
  __recola1_interface_rcl_MOD_get_alpha_rcl(&a);
}
ivoid set_alphas_rcl
      (const double a, const double s, const int nf)
{
  __recola1_interface_rcl_MOD_set_alphas_rcl(&a,&s,&nf);
}
ivoid get_alphas_rcl
      (double &a)
{
  __recola1_interface_rcl_MOD_get_alphas_rcl(&a);
}
ivoid set_alphas_masses_rcl
      (const double mc, const double mb, const double mt,
       const double wc, const double wb, const double wt)
{
  __recola1_interface_rcl_MOD_set_alphas_masses_rcl(&mc,&mb,&mt,&wc,&wb,&wt);
}
ivoid compute_running_alphas_rcl
      (const double q, const int nf, const int lp)
{
  __recola1_interface_rcl_MOD_compute_running_alphas_rcl(&q,&nf,&lp);
}
ivoid select_all_gs_powers_BornAmpl_rcl
      (const int npr)
{
  __recola1_interface_rcl_MOD_select_all_gs_powers_bornampl_rcl(&npr);
}
ivoid unselect_all_gs_powers_BornAmpl_rcl
      (const int npr)
{
  __recola1_interface_rcl_MOD_unselect_all_gs_powers_bornampl_rcl(&npr);
}
ivoid select_gs_power_BornAmpl_rcl
      (const int npr, const int gspower)
{
  __recola1_interface_rcl_MOD_select_gs_power_bornampl_rcl(&npr,&gspower);
}
ivoid unselect_gs_power_BornAmpl_rcl
      (const int npr, const int gspower)
{
  __recola1_interface_rcl_MOD_unselect_gs_power_bornampl_rcl(&npr,&gspower);
}
ivoid select_all_gs_powers_LoopAmpl_rcl
      (const int npr)
{
  __recola1_interface_rcl_MOD_select_all_gs_powers_loopampl_rcl(&npr);
}
ivoid unselect_all_gs_powers_LoopAmpl_rcl
      (const int npr)
{
  __recola1_interface_rcl_MOD_unselect_all_gs_powers_loopampl_rcl(&npr);
}
ivoid select_gs_power_LoopAmpl_rcl
      (const int npr, const int gspower)
{
  __recola1_interface_rcl_MOD_select_gs_power_loopampl_rcl(&npr,&gspower);
}
ivoid unselect_gs_power_LoopAmpl_rcl
      (const int npr, const int gspower)
{
  __recola1_interface_rcl_MOD_unselect_gs_power_loopampl_rcl(&npr,&gspower);
}
ivoid get_colour_correlation_rcl
      (const int npr, 
       const int pow, 
       const int i1, const int i2, 
       double& A2cc)
{
  __recola1_interface_rcl_MOD_get_colour_correlation_r1_rcl(&npr,
                                                            &pow,
                                                            &i1,
                                                            &i2,
                                                            &A2cc);
}
ivoid get_spin_colour_correlation_rcl
      (const int npr, 
       const int pow, 
       const int i1, const int i2, 
       double &A2scc)
{
  __recola1_interface_rcl_MOD_get_spin_colour_correlation_r1_rcl(&npr,
                                                                 &pow,
                                                                 &i1,
                                                                 &i2,
                                                                 &A2scc);
}
ivoid get_spin_correlation_rcl
      (const int npr,
       const int pow,
       double &A2sc)
{
  __recola1_interface_rcl_MOD_get_spin_correlation_r1_rcl(&npr,
                                                          &pow,
                                                          &A2sc);
}

/***********************************
*  module process_definition_rcl  *
***********************************/

ivoid define_process_rcl
      (const int npr,
       const std::string process, 
       const std::string order)
{
  __process_definition_rcl_MOD_define_process_rcl(&npr,
                                                  process.c_str(),
                                                  order.c_str(),
                                                  process.length(),
                                                  order.length());
}
ivoid select_power_BornAmpl_rcl
      (const int npr,
       const std::string orderid, 
       const int pow)
{
  __process_definition_rcl_MOD_select_power_bornampl_rcl(&npr, 
                                                         orderid.c_str(),
                                                         &pow,
                                                         orderid.length());
}
ivoid unselect_power_BornAmpl_rcl
      (const int npr,
       const std::string orderid, 
       const int pow)
{
  __process_definition_rcl_MOD_unselect_power_bornampl_rcl(&npr, 
                                                           orderid.c_str(),
                                                           &pow,
                                                           orderid.length());
}
ivoid select_all_powers_BornAmpl_rcl
      (const int npr)
{
  __process_definition_rcl_MOD_select_all_powers_bornampl_rcl(&npr);
}
ivoid unselect_all_powers_BornAmpl_rcl
      (const int npr)
{
  __process_definition_rcl_MOD_unselect_all_powers_bornampl_rcl(&npr);
}
ivoid select_power_LoopAmpl_rcl
      (const int npr,
       const std::string orderid, 
       const int pow)
{
  __process_definition_rcl_MOD_select_power_loopampl_rcl(&npr, 
                                                         orderid.c_str(),
                                                         &pow,
                                                         orderid.length());
}
ivoid unselect_power_LoopAmpl_rcl
      (const int npr,
       const std::string orderid, 
       const int pow)
{
  __process_definition_rcl_MOD_unselect_power_loopampl_rcl(&npr, 
                                                           orderid.c_str(),
                                                           &pow,
                                                           orderid.length());
}
ivoid select_all_powers_LoopAmpl_rcl
      (const int npr)
{
  __process_definition_rcl_MOD_select_all_powers_loopampl_rcl(&npr);
}
ivoid unselect_all_powers_LoopAmpl_rcl
      (const int npr)
{
  __process_definition_rcl_MOD_unselect_all_powers_loopampl_rcl(&npr);
}
ivoid split_collier_cache_rcl
      (const int npr,
       const int n)
{
  __process_definition_rcl_MOD_split_collier_cache_rcl(&npr,&n);
}

/***********************************
*  module process_generation_rcl  *
***********************************/

ivoid generate_processes_rcl()
{
  __process_generation_rcl_MOD_generate_processes_rcl();
}

/************************************
*  module process_computation_rcl  *
************************************/

ivoid set_resonant_squared_momentum_rcl
      (const int npr,const int res, const double ps)
{
  __process_computation_rcl_MOD_set_resonant_squared_momentum_rcl(&npr,&res,&ps);
}
ivoid rescale_process_rcl
      (const int npr, const std::string order, double A2[2])
{
  __process_computation_rcl_MOD_rescale_process_rcl(&npr,
                                                    order.c_str(),
                                                    A2,
                                                    order.length());
}
ivoid rescale_process_rcl
      (const int npr, const std::string order)
{
  double A2[2];
  __process_computation_rcl_MOD_rescale_process_rcl(&npr,
                                                    order.c_str(),
                                                    A2,
                                                    order.length());
}
ivoid rescale_colour_correlation_rcl
      (const int npr, 
       const int i1, const int i2, 
       double &A2cc)
{
  __process_computation_rcl_MOD_rescale_colour_correlation_rcl(&npr,
                                                               &i1,
                                                               &i2,
                                                               &A2cc);
}
ivoid rescale_colour_correlation_rcl
      (const int npr, 
       const int i1, const int i2)
{
  double A2cc;
  __process_computation_rcl_MOD_rescale_colour_correlation_rcl(&npr,
                                                               &i1,
                                                               &i2,
                                                               &A2cc);
}
ivoid rescale_all_colour_correlations_rcl
      (const int npr)
{
  __process_computation_rcl_MOD_rescale_all_colour_correlations_rcl(&npr);
}
ivoid rescale_spin_colour_correlation_rcl
      (const int npr, 
       const int i1, const int i2,  
       const std::complex<double> v[4], 
       double &A2scc)
{
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __process_computation_rcl_MOD_rescale_spin_colour_correlation_rcl(&npr,
                                                                    &i1,
                                                                    &i2,
                                                                    dfac,
                                                                    &A2scc);
}
ivoid rescale_spin_colour_correlation_rcl
      (const int npr, 
       const int i1, const int i2,  
       const std::complex<double> v[4])
{
  double A2scc;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __process_computation_rcl_MOD_rescale_spin_colour_correlation_rcl(&npr,
                                                                    &i1,
                                                                    &i2,
                                                                    dfac,
                                                                    &A2scc);
}
ivoid rescale_spin_correlation_rcl
      (const int npr, 
       const int j, 
       const std::complex<double> v[4],
       double &A2sc)
{
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __process_computation_rcl_MOD_rescale_spin_correlation_rcl(&npr,
                                                             &j,
                                                             dfac,
                                                             &A2sc);
}
ivoid rescale_spin_correlation_rcl
      (const int npr, 
       const int j, 
       const std::complex<double> v[4])
{
  double A2sc;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __process_computation_rcl_MOD_rescale_spin_correlation_rcl(&npr,
                                                             &j,
                                                             dfac,
                                                             &A2sc);
}
ivoid set_tis_required_accuracy_rcl
      (const double acc)
{
  __process_computation_rcl_MOD_set_tis_required_accuracy_rcl(&acc);
}
ivoid get_tis_required_accuracy_rcl
      (double &acc)
{
  __process_computation_rcl_MOD_get_tis_required_accuracy_rcl(&acc);
}
ivoid set_tis_critical_accuracy_rcl
      (const double acc)
{
  __process_computation_rcl_MOD_set_tis_critical_accuracy_rcl(&acc);
}
ivoid get_tis_critical_accuracy_rcl
      (double &acc)
{
  __process_computation_rcl_MOD_get_tis_critical_accuracy_rcl(&acc);
}
ivoid get_tis_accuracy_flag_rcl
      (int &flag)
{
  __process_computation_rcl_MOD_get_tis_accuracy_flag_rcl(&flag);
}


/********************
*  module wrapper  *
********************/
ivoid set_alphas_masses_rcl
       (const double mc, const double mb, const double mt)
{
  __wrapper_rcl_MOD_set_alphas_masses_nowidtharg_rcl(&mc,&mb,&mt);
}
ivoid use_gfermi_scheme_rcl()
{
  __wrapper_rcl_MOD_use_gfermi_scheme_noarg_rcl();
}
ivoid use_gfermi_scheme_and_set_alpha_rcl
      (const double a)
{
  __wrapper_rcl_MOD_use_gfermi_scheme_and_set_alpha_rcl(&a);
}
ivoid use_gfermi_scheme_and_set_gfermi_rcl
      (const double g)
{
  __wrapper_rcl_MOD_use_gfermi_scheme_and_set_gfermi_rcl(&g);
}
ivoid use_alpha0_scheme_rcl()
{
  __wrapper_rcl_MOD_use_alpha0_scheme_noarg_rcl();
}
ivoid use_alphaz_scheme_rcl()
{
  __wrapper_rcl_MOD_use_alphaz_scheme_noarg_rcl();
}
ivoid set_gs_power_rcl
      (const int npr,
       const int gsarray[][2],
       const int gslen)
{
   __wrapper_rcl_MOD_wrapper_set_gs_power_rcl(&npr,gsarray,&gslen);
}
ivoid set_outgoing_momenta_rcl
     (const int npr, double pIn[][4], double p[][4])
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  __wrapper_rcl_MOD_wrapper_set_outgoing_momenta_rcl(&npr, pIn, p, &legs);
}
ivoid compute_process_rcl
      (const int npr, 
       const double p[][4],
       const std::string order, 
       double A2[2], 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_process_rcl(&npr,
                                                p,
                                                &legs,
                                                order.c_str(),
                                                A2,
                                                &boolint,
                                                order.length());
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_process_rcl
      (const int npr, 
       const double p[][4],
       const std::string order, 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  double A2[2];
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_process_rcl(&npr,
                                                p,
                                                &legs,
                                                order.c_str(),
                                                A2,
                                                &boolint,
                                                order.length());
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_process_rcl
      (const int npr, 
       const double p[][4],
       const std::string order, 
       double A2[2])
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_process_rcl(&npr,
                                                p,
                                                &legs,
                                                order.c_str(),
                                                A2,
                                                &boolint,
                                                order.length());
}
ivoid compute_process_rcl
      (const int npr, 
       const double p[][4],
       const std::string order)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  double A2[2];
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_process_rcl(&npr,
                                                p,
                                                &legs,
                                                order.c_str(),
                                                A2,
                                                &boolint,
                                                order.length());
}
ivoid get_amplitude_rcl
      (const int npr, 
       const int pow, 
       const std::string order, 
       const int colour[], const int hel[],
       std::complex<double>& A)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  dcomplex dA;
  dA.di = A.imag();
  dA.dr = A.real();
  __wrapper_rcl_MOD_wrapper_get_amplitude_r1_rcl(&npr,
                                                 &pow,
                                                 order.c_str(), 
                                                 colour,
                                                 hel,
                                                 &legs,
                                                 &dA,
                                                 order.length());
  A = std::complex<double> (dA.dr, dA.di);
}
ivoid get_amplitude_rcl
      (const int npr, 
       const int pow[], 
       const std::string order, 
       const int colour[], const int hel[],
       std::complex<double>& A)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr,&legs);
  int powlen;
  __wrapper_rcl_MOD_get_n_orders_rcl(&powlen);
  dcomplex dA;
  dA.di = A.imag();
  dA.dr = A.real();
  __wrapper_rcl_MOD_wrapper_get_amplitude_general_rcl(&npr,
                                                      pow,
                                                      &powlen,
                                                      order.c_str(), 
                                                      colour,
                                                      hel,
                                                      &legs,
                                                      &dA,
                                                      order.length());
  A = std::complex<double> (dA.dr, dA.di);
}
ivoid get_squared_amplitude_rcl
      (const int npr, 
       const int pow, 
       const std::string order, 
       double &A2)
{
  __wrapper_rcl_MOD_wrapper_get_squared_amplitude_r1_rcl(&npr,
                                                         &pow,
                                                         order.c_str(),
                                                         &A2,
                                                         order.length());
}
ivoid get_squared_amplitude_rcl
      (const int npr, 
       const int pow[], 
       const std::string order, 
       double &A2)
{
  int powlen;
  __wrapper_rcl_MOD_get_n_orders_rcl(&powlen);
  __wrapper_rcl_MOD_wrapper_get_squared_amplitude_general_rcl(&npr,
                                                              pow,
                                                              &powlen,
                                                              order.c_str(),
                                                              &A2,
                                                              order.length());
}
ivoid get_polarized_squared_amplitude_rcl
      (const int npr, 
       const int pow, 
       const std::string order,
       const int hel[], 
       double &A2h)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  __wrapper_rcl_MOD_wrapper_get_polarized_squared_amplitude_r1_rcl(&npr,
                                                                   &pow,
                                                                   order.c_str(),
                                                                   hel,
                                                                   &legs,
                                                                   &A2h,
                                                                   order.length());
}
ivoid get_polarized_squared_amplitude_rcl
      (const int npr, 
       const int pow[], 
       const std::string order,
       const int hel[], 
       double &A2h)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int powlen;
  __wrapper_rcl_MOD_get_n_orders_rcl(&powlen);
  __wrapper_rcl_MOD_wrapper_get_polarized_squared_amplitude_general_rcl(&npr,
                                                                        pow,
                                                                        &powlen,
                                                                        order.c_str(),
                                                                        hel,
                                                                        &legs,
                                                                        &A2h,
                                                                        order.length());
}
ivoid compute_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       double &A2cc, 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_colour_correlation_rcl(&npr,
                                                           p,
                                                           &legs,
                                                           &i1,
                                                           &i2,
                                                           &A2cc,
                                                           &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  double A2cc;
  __wrapper_rcl_MOD_wrapper_compute_colour_correlation_rcl(&npr,
                                                           p,
                                                           &legs,
                                                           &i1,
                                                           &i2,
                                                           &A2cc,
                                                           &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       double &A2cc)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_colour_correlation_rcl(&npr,
                                                           p,
                                                           &legs,
                                                           &i1,
                                                           &i2,
                                                           &A2cc,
                                                           &boolint);
}
ivoid compute_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  double A2cc;
  __wrapper_rcl_MOD_wrapper_compute_colour_correlation_rcl(&npr,
                                                           p,
                                                           &legs,
                                                           &i1,
                                                           &i2,
                                                           &A2cc,
                                                           &boolint);
}
ivoid compute_all_colour_correlations_rcl
      (const int npr, 
       const double p[][4], 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_all_colour_correlations_rcl(&npr,
                                                                p,
                                                                &legs,
                                                                &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_all_colour_correlations_rcl
      (const int npr, 
       const double p[][4])
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_all_colour_correlations_rcl(&npr,
                                                                p,
                                                                &legs,
                                                                &boolint);
}
ivoid get_colour_correlation_rcl
      (const int npr, 
       int pow[], 
       const int i1, const int i2, 
       double& A2cc)
{

  int powlen;
  __wrapper_rcl_MOD_get_n_orders_rcl(&powlen);
  __wrapper_rcl_MOD_wrapper_get_colour_correlation_general_rcl(&npr,
                                                               pow,
                                                               &powlen,
                                                               &i1,
                                                               &i2,
                                                               &A2cc);
}
ivoid compute_spin_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       const std::complex<double> v[4], 
       double &A2scc, 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_spin_colour_correlation_rcl(&npr,
                                                                p,
                                                                &legs,
                                                                &i1,
                                                                &i2,
                                                                dfac,
                                                                &A2scc,
                                                                &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_spin_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       const std::complex<double> v[4], 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  double A2scc;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_spin_colour_correlation_rcl(&npr,
                                                                p,
                                                                &legs,
                                                                &i1,
                                                                &i2,
                                                                dfac,
                                                                &A2scc,
                                                                &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_spin_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       const std::complex<double> v[4], 
       double &A2scc)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __wrapper_rcl_MOD_wrapper_compute_spin_colour_correlation_rcl(&npr,
                                                                p,
                                                                &legs,
                                                                &i1,
                                                                &i2,
                                                                dfac,
                                                                &A2scc,
                                                                &boolint);
}
ivoid compute_spin_colour_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int i1, const int i2, 
       const std::complex<double> v[4])
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  double A2scc;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __wrapper_rcl_MOD_wrapper_compute_spin_colour_correlation_rcl(&npr,
                                                                p,
                                                                &legs,
                                                                &i1,
                                                                &i2,
                                                                dfac,
                                                                &A2scc,
                                                                &boolint);
}
ivoid get_spin_colour_correlation_rcl
      (const int npr, 
       int pow[], 
       const int i1, const int i2, 
       double &A2scc)
{
  int powlen;
  __wrapper_rcl_MOD_get_n_orders_rcl(&powlen);
  __wrapper_rcl_MOD_wrapper_get_spin_colour_correlation_general_rcl(&npr,
                                                                    pow,
                                                                    &powlen,
                                                                    &i1,
                                                                    &i2,
                                                                    &A2scc);
}
ivoid compute_spin_correlation_rcl
      (const int npr, 
       const double p[][4],
       const int j, 
       const std::complex<double> v[4],
       double &A2sc, 
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_spin_correlation_rcl(&npr,
                                                         p,
                                                         &legs,
                                                         &j,
                                                         dfac,
                                                         &A2sc,
                                                         &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_spin_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int j, 
       const std::complex<double> v[4],
       bool& momenta_check)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  double A2sc; 
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  int boolint;
  __wrapper_rcl_MOD_wrapper_compute_spin_correlation_rcl(&npr,
                                                         p,
                                                         &legs,
                                                         &j,
                                                         dfac,
                                                         &A2sc,
                                                         &boolint);
  if (boolint) {
    momenta_check = true;
  } else {
    momenta_check = false;
  }
}
ivoid compute_spin_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int j, 
       const std::complex<double> v[4],
       double &A2sc)
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __wrapper_rcl_MOD_wrapper_compute_spin_correlation_rcl(&npr,
                                                         p,
                                                         &legs,
                                                         &j,
                                                         dfac,
                                                         &A2sc,
                                                         &boolint);
}
ivoid compute_spin_correlation_rcl
      (const int npr, 
       const double p[][4], 
       const int j, 
       const std::complex<double> v[4])
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  int boolint;
  double A2sc;
  dcomplex dfac[4];
  for (int i = 0; i < 4; ++i) {
    dfac[i].dr = v[i].real();
    dfac[i].di = v[i].imag();
  }
  __wrapper_rcl_MOD_wrapper_compute_spin_correlation_rcl(&npr,
                                                         p,
                                                         &legs,
                                                         &j,
                                                         dfac,
                                                         &A2sc,
                                                         &boolint);
}
ivoid get_spin_correlation_rcl
      (const int npr,
       int pow[],
       double &A2sc)
{
  int powlen;
  __wrapper_rcl_MOD_get_n_orders_rcl(&powlen);
  __wrapper_rcl_MOD_wrapper_get_spin_correlation_general_rcl(&npr,
                                                             pow,
                                                             &powlen,
                                                             &A2sc);
}
// get_legs_rcl
// get_n_orders_rcl
ivoid get_momenta_rcl
      (const int npr, double p[][4])
{
  int legs;
  __wrapper_rcl_MOD_get_legs_rcl(&npr, &legs);
  __wrapper_rcl_MOD_wrapper_get_momenta_rcl(&npr,p,&legs);
}
ivoid get_recola_version_rcl
      (std::string& recolaversion)
{
  char rv_cstr[10];
  int len;
  __wrapper_rcl_MOD_wrapper_get_recola_version_rcl(rv_cstr,&len,10);
  recolaversion = std::string(rv_cstr,len);
}
ivoid get_modelname_rcl
      (std::string& modelname)
{
  char modelname_cstr[100];
  int len;
  __wrapper_rcl_MOD_wrapper_get_modelname_rcl(modelname_cstr,&len,100);
  modelname = std::string(modelname_cstr,len);
}
ivoid get_modelgauge_rcl
      (std::string& modelgauge)
{
  char modelgauge_cstr[100];
  int len;
  __wrapper_rcl_MOD_wrapper_get_modelgauge_rcl(modelgauge_cstr,&len,100);
  modelgauge = std::string(modelgauge_cstr,len);
}
ivoid get_driver_timestamp_rcl
      (std::string& driverts)
{
  char driverts_cstr[200];
  int len;
  __wrapper_rcl_MOD_wrapper_get_driver_timestamp_rcl(driverts_cstr,&len,200);
  driverts = std::string(driverts_cstr,len);
}
ivoid get_renoscheme_rcl
      (const std::string paramname, std::string& renoscheme)
{

  char renoscheme_cstr[100];
  int len;
  __wrapper_rcl_MOD_wrapper_get_renoscheme_rcl(paramname.c_str(),
                                               renoscheme_cstr,&len,100);
  renoscheme = std::string(renoscheme_cstr,len);
}

/*****************************************
*  module extended_higgs_interface_rcl  *
*****************************************/

ivoid set_Z2_thdm_yukawa_type_rcl
      (const int ytype)
{
  __extended_higgs_interface_rcl_MOD_set_z2_thdm_yukawa_type_rcl(&ytype);
}
ivoid set_pole_mass_hl_rcl
      (const double ml,
       const double gl)
{
  __extended_higgs_interface_rcl_MOD_set_pole_mass_hl_rcl(&ml,&gl);
}
ivoid set_pole_mass_hh_rcl
      (const double mh,
       const double gh)
{
  __extended_higgs_interface_rcl_MOD_set_pole_mass_hh_rcl(&mh,&gh);
}
ivoid set_pole_mass_hl_hh_rcl
      (const double ml,
       const double gl,
       const double mh,
       const double gh)
{
  __extended_higgs_interface_rcl_MOD_set_pole_mass_hl_hh_rcl(&ml,&gl,&mh,&gh);
}
ivoid set_pole_mass_ha_rcl
      (const double ma,
       const double ga)
{
  __extended_higgs_interface_rcl_MOD_set_pole_mass_ha_rcl(&ma,&ga);
}
ivoid set_pole_mass_hc_rcl
      (const double hc,
       const double gc)
{
  __extended_higgs_interface_rcl_MOD_set_pole_mass_hc_rcl(&hc,&gc);
}
ivoid set_tb_cab_rcl
      (const double tb,
       const double cab)
{
   __extended_higgs_interface_rcl_MOD_set_tb_cab_rcl(&tb,&cab);
}
ivoid use_mixing_alpha_msbar_scheme_rcl
      (const std::string s)
{
   __extended_higgs_interface_rcl_MOD_use_mixing_alpha_msbar_scheme_rcl(s.c_str(), s.length());
}
ivoid use_mixing_alpha_onshell_scheme_rcl
      (const std::string s)
{
   __extended_higgs_interface_rcl_MOD_use_mixing_alpha_onshell_scheme_rcl(s.c_str(), s.length());
}
ivoid use_mixing_beta_onshell_scheme_rcl
      (const std::string s)
{
   __extended_higgs_interface_rcl_MOD_use_mixing_beta_onshell_scheme_rcl(s.c_str(), s.length());
}
ivoid use_mixing_beta_msbar_scheme_rcl
      (const std::string s)
{
   __extended_higgs_interface_rcl_MOD_use_mixing_beta_msbar_scheme_rcl(s.c_str(), s.length());
}
ivoid set_msb_rcl
      (const double msb)
{
   __extended_higgs_interface_rcl_MOD_set_msb_rcl(&msb);
}
ivoid use_msb_msbar_scheme_rcl
      (const std::string s)
{
   __extended_higgs_interface_rcl_MOD_use_msb_msbar_scheme_rcl(s.c_str(), s.length());
}
ivoid set_sa_rcl
      (const double sa)
{
  __extended_higgs_interface_rcl_MOD_set_sa_rcl(&sa);
}
ivoid set_tb_rcl
      (const double tb)
{
  __extended_higgs_interface_rcl_MOD_set_tb_rcl(&tb);
}
ivoid use_tb_msbar_scheme_rcl
      (const std::string s)
{
   __extended_higgs_interface_rcl_MOD_use_tb_msbar_scheme_rcl(s.c_str(), s.length());
}

/***************************
*  module statistics_rcl  *
***************************/

ivoid print_generation_statistics_rcl
      ()
{
  __statistics_rcl_MOD_print_generation_statistics_rcl();
}
ivoid print_TI_statistics_rcl
      (const int npr, const int npoints)
{
  __statistics_rcl_MOD_print_ti_statistics_rcl(&npr, &npoints);
}
ivoid print_TC_statistics_rcl
      (const int npr, const int npoints)
{
  __statistics_rcl_MOD_print_tc_statistics_rcl(&npr, &npoints);
}

/**********************
*  module reset_rcl  *
**********************/

ivoid reset_recola_rcl()
{
  __reset_rcl_MOD_reset_recola_rcl();
}

}

#endif // RECOLA_HPP_
