!******************************************************************************!
!                                                                              !
!    check_rcl.f90                                                             !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016,2017 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!


  module check_rcl

!------------------------------------------------------------------------------!

  use input_rcl
  use recola1_interface_rcl
  use process_definition_rcl
  use process_generation_rcl
  use process_computation_rcl
  use reset_rcl

!------------------------------------------------------------------------------!

  implicit none

!------------------------------------------------------------------------------!

  contains

!------------------------------------------------------------------------------!

  subroutine check_process_rcl (processIn,delta)

    character(len=*), intent(in)            :: processIn
    real(dp),         intent(out), optional :: delta

    integer               :: i,n,legs,gs0,gs1
    integer, allocatable  :: gs(:,:)
    real(dp)              :: p4a(0:3,4),matrix2Psum(0:1),mat2r,mat2r_tmp,mat2c
    real(dp), allocatable :: p(:,:),matrix2P(:,:)
    character             :: order*3,check*10,cpr*99,           &
                             fmt1i*99,fmt2i*99,fmt1*99,fmt2*99, &
                             fmt(0:3)*6,fmtTot*80
    character(len=100)    :: modelgauge

    ! Corrections may be gauge-dependent due to the expansion performed in the
    ! CMS (gauge-dependence is formally of higher order).
    ! Therefore, some tests are done based on the given gauge.
    call get_modelgauge_rcl(modelgauge)

    call set_pole_mass_z_rcl(91.154892493050440d0,2.4421237356890875d0)
    call set_pole_mass_w_rcl(80.371978311278895d0,2.0842992422669000d0)
    call set_pole_mass_h_rcl(120d0,0d0)
    call set_pole_mass_top_rcl(172.6d0,0d0)

    call set_mu_uv_rcl(80.371978311278895d0)
    call set_mu_ir_rcl(80.371978311278895d0)
    call use_gfermi_scheme_rcl(g=1.16637d-5)

    als = 0.12027511899498215d0
    Qren = 80.371978311278895d0
    Nfren = 5
    call set_alphas_rcl(0.12027511899498215d0,80.371978311278895d0,5)

    resSE = .false.

    momenta_correction = .true.

    colour_optimization = 2
    fermion_loop_opt = .true.
    writeMat  = 0
    writeMat2 = 2
    writeCor  = 0
    draw      = 0

    ifail = 0

    order = 'NLO'

    gs0 = 111
    gs1 = 111

    p4a(0,1) = + 4000d0
    p4a(1,1) =   0d0
    p4a(2,1) =   0d0
    p4a(3,1) = + 4000d0
    p4a(0,2) = + 4000d0
    p4a(1,2) =   0d0
    p4a(2,2) =   0d0
    p4a(3,2) = - 4000d0
    p4a(0,3) = + 4000d0
    p4a(1,3) = + 2144.3833090906010d0
    p4a(2,3) = + 3373.7194807643473d0
    p4a(3,3) = + 140.13239741325378d0
    p4a(0,4) = + 4000d0
    p4a(1,4) = - 2144.3833090906010d0
    p4a(2,4) = - 3373.7194807643473d0
    p4a(3,4) = - 140.13239741325378d0

    ! process

    cpr = adjustl(processIn)//'END'
    n = 99
    do while (index (cpr,'  ').ne.index (cpr,'END')+3)
      n = index (cpr,'  ')
      cpr = cpr(1:n-1)//cpr(n+1:)
    enddo
    n = 99
    do while (index (cpr,' [').ne.0)
      n = index (cpr,' [')
      cpr = cpr(1:n-1)//cpr(n+1:)
    enddo
    n = 99
    do while (index (cpr,' (').ne.0)
      n = index (cpr,' (')
      cpr = cpr(1:n-1)//cpr(n+1:)
    enddo
    n = 99
    do while (index (cpr,'( ').ne.0)
      n = index (cpr,'( ')
      cpr = cpr(1:n)//cpr(n+2:)
    enddo
    n = 99
    do while (index (cpr,' )').ne.0)
      n = index (cpr,' )')
      cpr = cpr(1:n-1)//cpr(n+1:)
    enddo
    n = index (cpr,'END')
    cpr = cpr(1:n-1)

    matrix2Psum = 0d0

    if (cpr.eq.'u~ u -> nu_e~ nu_e') then

      check = 'Pole'
      legs = 4
      allocate (gs(0:legs,0:1)); gs = 0; gs(0,0:1) = 1
      allocate(p(0:3,legs)); p = p4a
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(0,0) = 9.57814610849876732d-004
      matrix2P(0,1) = &
      - 7.85646307274683107d-004 & ! boxes
      + 2.48717467405677338d-004 & ! triangles
      + 4.01206502134965519d-005   ! self-energies
      !- 7.38975225297194724d-005   ! original counterterms, missing IR R2

      if (modelgauge .eq. "'t Hooft-Feynman") then
        matrix2P(0,1) = matrix2P(0,1) + 1.7889887323681514d-006
      else if (modelgauge .eq. "'t Hooft-Feynman BFM") then
        matrix2P(0,1) = matrix2P(0,1) + 1.7801472714799411d-006
      end if

    elseif (cpr.eq.'u d~ -> nu_e e+') then

      check = 'Pole'
      legs = 4
      allocate (gs(0:legs,0:1)); gs = 0; gs(0,0:1) = 1
      allocate(p(0:3,legs)); p = p4a
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(0,0) = 4.05436322743390424d-003
      matrix2P(0,1) = &
      - 3.34685213428006145d-003 & ! boxes        
      + 9.54435703234285480d-004 & ! triangles    
      + 1.63047377548910415d-004   ! self-energies
      !- 3.30426293431707518d-004   ! original counterterms, missing IR R2

      ! Recola2 ct
      if (modelgauge .eq. "'t Hooft-Feynman") then
        matrix2P(0,1) = matrix2P(0,1) - 1.0050508182941529d-005
      else if (modelgauge .eq. "'t Hooft-Feynman BFM") then
        matrix2P(0,1) = matrix2P(0,1) - 1.0093403923622387d-005
      end if


    elseif (cpr.eq.'d~ d -> u~ u') then

      check = 'Pole'
      legs = 4
      allocate (gs(0:legs,0:1)); gs = 1
      allocate(p(0:3,legs)); p = p4a
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2Psum(0) = 0.41284064171020179d0
      !matrix2Psum(1) = -1.4881832162850928d0   

      ! Recola2 result
      if (modelgauge .eq. "'t Hooft-Feynman") then
        matrix2Psum(1) = -1.4894315468811770d0
      else if (modelgauge .eq. "'t Hooft-Feynman BFM") then
        matrix2Psum(1) = -1.4894313782888620d0
      end if


    elseif (cpr.eq.'e+ e- -> nu_e~ nu_e') then

      check = 'Pole'
      legs = 4
      allocate (gs(0:legs,0:1)); gs = 1
      allocate(p(0:3,legs)); p = p4a
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2Psum(0) = 7.26179240617077365d-2
      !matrix2Psum(1) = -4.70818596988321139d-2

      ! Recola2 result
      if (modelgauge .eq. "'t Hooft-Feynman") then
        matrix2Psum(1) = -4.1343591285439386d-002  !Recola2 result
      else if (modelgauge .eq. "'t Hooft-Feynman BFM") then
        matrix2Psum(1) = -4.1344466580766584d-002
      end if

    elseif (cpr.eq.'e+ e- -> W+ W-') then

      check = 'Pole'
      legs = 4
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_mu_uv_rcl(80.399d0)
      call set_mu_ir_rcl(80.399d0)
      call set_mu_ms_rcl(80.399d0)
      allocate (gs(0:legs,0:1)); gs = 1
      allocate(p(0:3,legs))
      p(0,1) = + 4000d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 4000d0
      p(0,2) = + 4000d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 4000d0
      p(0,3) = + 4000d0
      p(1,3) = + 2143.9500999571701d0
      p(2,3) = + 3373.0379206689554d0
      p(3,3) = + 140.10408781291710d0
      p(0,4) = + 4000d0
      p(1,4) = - 2143.9500999571701d0
      p(2,4) = - 3373.0379206689554d0
      p(3,4) = - 140.10408781291710d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2Psum(0) = 2.96674776566679364d-2
      matrix2Psum(1) = -3.40773328495682926d-2

    elseif (cpr.eq.'u d~ -> W+ H') then

      check = 'Pole'
      legs = 4
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_mu_uv_rcl(80.399d0)
      call set_mu_ir_rcl(80.399d0)
      call set_mu_ms_rcl(80.399d0)
      allocate (gs(0:legs,0:1)); gs = 1
      allocate(p(0:3,legs))
      p(0,1) = + 4000d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 4000d0
      p(0,2) = + 4000d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 4000d0
      p(0,3) = + 3999.5039999500627d0
      p(1,3) = + 2143.6841426646142d0
      p(2,3) = + 3372.6194948701982d0
      p(3,3) = + 140.08670788235273d0
      p(0,4) = + 4000.4960000499377d0
      p(1,4) = - 2143.6841426646142d0
      p(2,4) = - 3372.6194948701982d0
      p(3,4) = - 140.08670788235273d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2Psum(0) = 1.86729421127388080d-3
      matrix2Psum(1) = -3.93591538157348972d-3

    elseif (cpr.eq.'e+ e- -> Z H') then

      check = 'Pole'
      legs = 4
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      allocate (gs(0:legs,0:1)); gs = 1
      allocate(p(0:3,legs))
      p(0,1) = + 4000d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 4000d0
      p(0,2) = + 4000d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 4000d0
      p(0,3) = + 3999.6196986496102d0
      p(1,3) = + 2143.6220891503181d0
      p(2,3) = + 3372.5218671983439d0
      p(3,3) = + 140.08265277350660d0
      p(0,4) = + 4000.3803013503903d0
      p(1,4) = - 2143.6220891503181d0
      p(2,4) = - 3372.5218671983439d0
      p(3,4) = - 140.08265277350660d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2Psum(0) = 2.37149351547372590d-3
      matrix2Psum(1) = -1.63698726254942113d-3

    elseif (cpr.eq.'u d~ -> W+ g') then

      check = 'openLoops'
      legs = 4
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_mu_ms_rcl(100d0)
      allocate (gs(0:legs,0:1)); gs = 0; gs(1,0) = 1; gs(3,1) = 1
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) = + 503.2319996005000462d0
      p(1,3) = + 302.3147316012482406d0
      p(2,3) = + 163.3466058461748389d0
      p(3,3) = + 358.7507987953716793d0
      p(0,4) = + 496.7680003995000106d0
      p(1,4) = - 302.3147316012482406d0
      p(2,4) = - 163.3466058461748105d0
      p(3,4) = - 358.7507987953716793d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(2,0) = 0.95821667535212d0
      matrix2P(4,1) = -0.15278750780672d0

    elseif (cpr.eq.'g g -> g g') then

      check = 'openLoops'
      legs = 4
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_mu_ms_rcl(100d0)
      allocate (gs(0:legs,0:1)); gs = 1
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) = + 500d0
      p(1,3) = + 304.28160767010678d0
      p(2,3) = + 164.4093477385940218d0
      p(3,3) = + 361.0848509836230278d0
      p(0,4) = + 500d0
      p(1,4) = - 304.28160767010678d0
      p(2,4) = - 164.4093477385940218d0
      p(3,4) = - 361.0848509836230278d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2Psum(0) = 0.24511877831620d3
      matrix2Psum(1) = -0.97387038866635d2

    elseif (cpr.eq.'b b~ -> t t~') then

      check = 'openLoops'
      legs = 4
      gs0 = 2
      gs1 = 4
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_pole_mass_bottom_rcl(5d0,0d0)
      call set_delta_ir_rcl(0d0,pi**2/6d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      allocate (gs(0:legs,0:1)); gs = 0
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 499.9749993749688d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 499.9749993749688d0
      p(0,3) =   500.0d0		      
      p(1,3) =   285.7111940687087d0   
      p(2,3) =   154.3753873858148d0   
      p(3,3) =   339.0477154521355d0   
      p(0,4) =   500.0d0		      
      p(1,4) = - 285.7111940687087d0  
      p(2,4) = - 154.37538738581478d0 
      p(3,4) = - 339.0477154521355d0  
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(gs0+gs0,0) = + 8.765797797519910d-01
      matrix2P(gs0+gs1,1) = + 8.652605119737087d-01

    elseif (cpr.eq.'g g -> b b~') then

      check = 'openLoops'
      legs = 4
      gs0 = 2
      gs1 = 4

      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_pole_mass_bottom_rcl(5d0,0d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      call set_delta_ir_rcl(0d0,pi**2/6d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      allocate (gs(0:legs,0:1)); gs = 0
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) = 500.0d0		     
      p(1,3) = 304.26639320935226d0  
      p(2,3) = 164.40112706568513d0  
      p(3,3) = 361.0667962896952d0   
      p(0,4) = 500.0d0		             
      p(1,4) = -304.26639320935226d0
      p(2,4) = -164.4011270656851d0 
      p(3,4) = -361.0667962896952d0 
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(gs0+gs0,0) = 1.936226265893901d+00
      matrix2P(gs0+gs1,1) = 1.173402037004956d+00

    elseif (cpr.eq.'u u~ -> W+ W- g') then

      check = 'openLoops'
      legs = 5
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      
      call set_delta_ir_rcl(0d0,pi**2/6d0)
      allocate (gs(0:legs,0:1)); gs = 0; gs(1,0) = 1; gs(3,1) = 1
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) = + 438.218295761793172d0
      p(1,3) = + 244.1807322977003309d0
      p(2,3) = + 153.8582132409753171d0
      p(3,3) = + 319.804152149755339d0
      p(0,4) = + 495.5814786225607236d0
      p(1,4) = - 299.1286500603476384d0
      p(2,4) = - 150.133911405357054d0
      p(3,4) = - 356.5373221865085611d0
      p(0,5) = +  66.200225615646147d0
      p(1,5) = +  54.947917762647144d0
      p(2,5) = -   3.7243018356182249d0
      p(3,5) = +  36.7331700367532932d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(2,0) = + 2.478709565073970d-04
      matrix2P(4,1) = - 2.605398472478095d-06

    elseif (cpr.eq.'u u~ -> Z Z g') then

      check = 'openLoops'
      legs = 5
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      allocate (gs(0:legs,0:1)); gs = 0; gs(1,0) = 1; gs(3,1) = 1
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) = + 417.4206246099386135d0
      p(1,3) = + 127.8541441091167314d0
      p(2,3) = - 194.2513616124706175d0
      p(3,3) = - 334.4316459335413469d0
      p(0,4) = + 155.8236098065406736d0
      p(1,4) = - 50.091070172690884d0
      p(2,4) = + 90.7786247395575003d0
      p(3,4) = - 72.2214992833172431d0
      p(0,5) = + 426.755765583520656d0
      p(1,5) = - 77.7630739364259114d0
      p(2,5) = + 103.4727368729131598d0
      p(3,5) = + 406.6531452168586611d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(2,0) = + 1.190560462998326d-05
      matrix2P(4,1) = - 8.870018925211744d-08

    elseif (cpr.eq.'u u~ -> Z A g') then

      check = 'openLoops'
      legs = 5
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      allocate (gs(0:legs,0:1)); gs = 0; gs(1,0) = 1; gs(3,1) = 1
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) = + 467.9220320217552853d0
      p(1,3) = - 161.2362660353179535d0
      p(2,3) = - 343.2253414521593413d0
      p(3,3) = - 258.5248172954496795d0
      p(0,4) = + 410.5097303987137138d0
      p(1,4) = +  76.9704063632674007d0
      p(2,4) = + 364.0379035243619228d0
      p(3,4) = + 173.407612560364754d0 
      p(0,5) = + 121.5682375795313988d0
      p(1,5) = +  84.2658596720505955d0
      p(2,5) = -  20.8125620722026383d0
      p(3,5) = +  85.1172047350850534d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(2,0) = 4.720966256967035d-06
      matrix2P(4,1) = 1.510513880716169d-07

    elseif (cpr.eq.'g g -> g t t~') then

      check = 'openLoops'
      legs = 5
      gs0 = 3
      gs1 = 5
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_delta_ir_rcl(0d0,pi**2/6d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      allocate (gs(0:legs,0:1)); gs = 0; gs(gs0,0) = 1; gs(5,1) = 1
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) =   367.60373422843884d0
      p(1,3) =   208.37035412957272d0
      p(2,3) =   131.29410366285185d0
      p(3,3) =   272.90320496831384d0
      p(0,4) =   451.35675954947516d0
      p(1,4) = - 255.25987311474162d0
      p(2,4) = - 128.11599011936772d0
      p(3,4) = - 304.2492639994103d0
      p(0,5) =   181.03950622208615d0
      p(1,5) =    46.88951898516874d0
      p(2,5) = -   3.1781135434841024d0
      p(3,5) =    31.34605903109653d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(gs0+gs0,0) = + 1.497698555532992d-3
      matrix2P(gs0+gs1,1) = - 5.145734157852298d-4

    elseif (cpr.eq.'u d~ -> W+ g g g') then

      check = 'openLoops'
      legs = 6
      gs0 = 3
      gs1 = 5
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_delta_ir_rcl(0d0,pi**2/6d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      allocate (gs(0:legs,0:1)); gs = 0
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) =   263.4845819918209d0  
      p(1,3) = -  20.526967070412418d0
      p(2,3) =   128.77196215622962d0 
      p(3,3) =   214.37479126923805d0 
      p(0,4) =   304.4026200608687d0  
      p(1,4) = - 300.63757474928536d0
      p(2,4) =    25.834297426690753d0 
      p(3,4) = -  40.13219188206375d0 
      p(0,5) =    26.69963460475724d0  
      p(1,5) =     6.683489195278001d0  
      p(2,5) =     5.616637251194153d0  
      p(3,5) =    25.23202025587033d0  
      p(0,6) =   405.41316334255316d0 
      p(1,6) =   314.4810526244197d0  
      p(2,6) = - 160.22289683411447d0
      p(3,6) = - 199.47461964304466d0
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(gs0+gs0,0) = + 1.791371971976059d-06
      matrix2P(gs0+gs1,1) = - 1.917092204236691d-06

    elseif (cpr.eq.'u u~ -> u u~ u u~') then

      check = 'openLoops'
      legs = 6
      gs0 = 4
      gs1 = 6
      call use_alphaZ_scheme_rcl(0.0078125d0)
      call set_pole_mass_z_rcl(91.1876d0,0d0)
      call set_pole_mass_w_rcl(80.399d0,0d0)
      call set_pole_mass_h_rcl(125d0,0d0)
      call set_pole_mass_top_rcl(172d0,0d0)
      call set_delta_ir_rcl(0d0,pi**2/6d0)
      call set_mu_uv_rcl(100d0)
      call set_mu_ir_rcl(100d0)
      call set_alphas_rcl(0.125808685692d0,100d0,5)
      allocate (gs(0:legs,0:1)); gs = 0
      allocate(p(0:3,legs))
      p(0,1) = + 500d0
      p(1,1) =   0d0
      p(2,1) =   0d0
      p(3,1) = + 500d0
      p(0,2) = + 500d0
      p(1,2) =   0d0
      p(2,2) =   0d0
      p(3,2) = - 500d0
      p(0,3) =   254.11173663453982d0 
      p(1,3) = -  20.78819186394802d0 
      p(2,3) =   130.41070543048184d0 
      p(3,3) =   217.10291035261022d0 
      p(0,4) =   308.2764271997574d0  
      p(1,4) = - 304.46346817638226d0
      p(2,4) =    26.163062947104724d0 
      p(3,4) = -  40.642911439539645d0
      p(0,5) =    27.039412347527474d0 
      p(1,5) =     6.768542826393844d0   
      p(2,5) =     5.688114196681263d0   
      p(3,5) =    25.55312123777459d0  
      p(0,6) =   410.57242381817514d0 
      p(1,6) =   318.4831172139364d0  
      p(2,6) = - 162.2618825742678d0 
      p(3,6) = - 202.0131201508452d0 
      allocate (matrix2P(0:2*legs-2,0:1)); matrix2P = 0d0
      matrix2P(gs0+gs0,0) = + 8.258218711346499d-07
      matrix2P(gs0+gs1,1) = - 3.861723516505142d-07

    else

      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) 'ERROR: No checked process'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)

    endif

    prTot = 0
    call define_process_rcl (1,cpr,order)

    call set_gs_power_rcl (1,gs(0:legs,0:1))

    if (gs0.ne.111) then
      call unselect_all_gs_powers_BornAmpl_rcl (1)
      call select_gs_power_BornAmpl_rcl (1,gs0)
    endif
    if (gs1.ne.111) then
      call unselect_all_gs_powers_LoopAmpl_rcl (1)
      call select_gs_power_LoopAmpl_rcl (1,gs1)
    endif

    call openOutput

    write(nx,*)
    write(nx,'(1x,75("*"))')
    write(nx,*)

    call generate_processes_rcl

    call compute_process_rcl (1,p,'NLO')

    write(nx,*)
    write(nx,'(1x,75("*"))')
    write(nx,*)

    write(nx,'(1x,75("-"))')

    write(nx,'(2x,a)') trim(cpr)

    write(nx,'(1x,75("-"))')

    do i = 1,legs
      fmt(0) = 'f15.10'
      if (abs(p(0,i)).ge.1d4) fmt(0) = 'e15.9'
      fmtTot = '(2x,a,i1,1x,a,1x,"(",'//fmt(0)
      do n = 1,3
        fmt(n) = 'f16.10'
        if (abs(p(n,i)).ge.1d4) fmt(n) = 'e16.9'
        fmtTot = trim(fmtTot)//',",",'//fmt(n)
      enddo
      fmtTot = trim(fmtTot)//',")")'
      write(nx,trim(fmtTot)) 'p',i,'=',p(0:3,i)
    enddo

    fmt1i = '(4x,i2," | ",e21.14,2x,a)'
    fmt2i = '(4x,i2," | ",e21.14,2x,a,5x,a,e21.14)'
    fmt1 = '(3x,a3," | ",e21.14,2x,a)'
    fmt2 = '(3x,a3," | ",e21.14,2x,a,5x,a,e21.14)'

    write(nx,'(1x,75("-"))')
    write(nx,*) '  als |        | A0 |^2      '
    do i = 0,legs-2
      if (matrix2P(2*i,0).ne.0d0) then
        call get_squared_amplitude_r1_rcl(1,i,'LO',mat2r)
        mat2c = matrix2P(2*i,0)
        delta = abs(mat2r-mat2c)/abs(mat2r+mat2c)*2
        write(nx,*) ' ========================================'
        write(nx,fmt1i) i/2,mat2r,'recola'
        write(nx,fmt2i) i/2,mat2c,check,'Delta =',delta
      endif
    enddo
    if (matrix2Psum(0).ne.0d0) then
      mat2r = 0d0
      do i = 0, legs-2
        call get_squared_amplitude_r1_rcl(1,i,'LO',mat2r_tmp)
        mat2r = mat2r + mat2r_tmp
      end do
      mat2c =     matrix2Psum(0)
      delta = abs(mat2r-mat2c)/abs(mat2r+mat2c)*2
      write(nx,*) ' ========================================'
      write(nx,fmt1) 'all',mat2r,'recola'
      write(nx,fmt2) 'all',mat2c, check,'Delta =',delta
    endif
    write(nx,'(1x,75("-"))')

    if (loop(1)) then
      write(nx,*) '  als |   2*Re{ A1 * A0^* }  '
      do i = 0,legs-1
        if (matrix2P(2*i,1).ne.0d0) then
          call get_squared_amplitude_r1_rcl(1,i,'NLO',mat2r)
          mat2c = matrix2P(2*i,1)
          delta = abs(mat2r-mat2c)/abs(mat2r+mat2c)*2
          write(nx,*) ' ========================================'
          write(nx,fmt1i) i,mat2r,'recola'
          write(nx,fmt2i) i,mat2c,check,'Delta =',delta
        endif
      enddo
      if (matrix2Psum(1).ne.0d0) then
        mat2r = 0d0
        do i = 0, 2*legs-2
          call get_squared_amplitude_r1_rcl(1,i,'NLO',mat2r_tmp)
          mat2r = mat2r + mat2r_tmp
        end do
        mat2c =     matrix2Psum(1)
        delta = abs(mat2r-mat2c)/abs(mat2r+mat2c)*2
        write(nx,*) ' ========================================'
        write(nx,fmt1) 'all',mat2r,'recola'
        write(nx,fmt2) 'all',mat2c,check,'Delta =',delta
      endif
      write(nx,'(1x,75("-"))')
    endif

    deallocate (matrix2P,p,gs)

    call reset_recola_rcl

  end subroutine check_process_rcl

!------------------------------------------------------------------------------!

  end module check_rcl

!------------------------------------------------------------------------------!
